/* Implementation of break, continue and others */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "Constant_p.h"




struct _BBCmdConstantCommandParameter {
	const char* name;
	int return_code;
};

static const BBCmdConstantCommandParameter PARAMETER_TABLE[] = {
	{"break",	BBCmd_RC_BREAK},
	{"continue",	BBCmd_RC_CONTINUE},
	{"false",	BBCmd_RC_FALSE},
	{"true",	BBCmd_RC_TRUE},
	{NULL,		0},
};

static const BBCmdConstantCommandParameter*
search_parameter(const char* name)
{
	const BBCmdConstantCommandParameter* p;

	for (p = PARAMETER_TABLE; p->name != NULL; p++) {
		if (!strcmp(name, p->name))
			return p;
	}

	return NULL;
}




static const char*
_BBCmdConstantCommand_initialize( BBCmdCommand* _self,
                                 int ac, const char* const* av,
                                 const BBCmdCompiler* compiler )
{
	BBCmdConstantCommand* self;
	const BBCmdConstantCommandParameter* parameter;

	/*assert(_self->cmd_descriptor == BBCmdConstantCommandDescriptor);*/
	self = (BBCmdConstantCommand*)_self;

	if (ac - 1 != 0) return "Incorrect number of arguments";

	parameter = search_parameter(av[0]);
	if (parameter == NULL) return "Unknown command (maybe BBCmd's bug)";

	self->co_return_code = parameter->return_code;

	return NULL;
}


static void
_BBCmdConstantCommand_del(BBCmdCommand* _self)
{
	/* nothing to do */
}


static int
_BBCmdConstantCommand_execute(const BBCmdCommand* self)
{
	return ((BBCmdConstantCommand*)self)->co_return_code;
}




#define DEFINE_DESC(NAME,DOC)					\
	static const BBCmdCommandDescriptor			\
			_BBCmd__##NAME##__CommandDescriptor={	\
		#NAME,						\
		NULL,						\
		DOC,						\
		sizeof(BBCmdConstantCommand),			\
		_BBCmdConstantCommand_initialize,		\
		_BBCmdConstantCommand_del,			\
		_BBCmdConstantCommand_execute			\
	};							\
	const BBCmdCommandDescriptor* const BBCmd__##NAME##__CommandDescriptor\
		= &_BBCmd__##NAME##__CommandDescriptor;


DEFINE_DESC(break,"()\nBreak loop or other commands.")
DEFINE_DESC(continue,
            "()\nContinue with the next cycle of the nearest enclosing loop.")
DEFINE_DESC(false, "() --> false\nReturn false.")
DEFINE_DESC(true, "() --> true\nReturn true.")




/* __END__ */
