/* Implementation of `exec' command */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>

#include "Exec_p.h"




static const char*
_BBCmdExecCommand_initialize( BBCmdCommand* _self,
                              int ac, const char* const* av,
                              const BBCmdCompiler* compiler )
{
	BBCmdExecCommand* self;
	int i;

	assert(_self->cmd_descriptor == BBCmdExecCommandDescriptor);
	self = (BBCmdExecCommand*)_self;

	self->ex_operation = NULL;
	self->ex_file = NULL;
	self->ex_parameters = NULL;
	self->ex_directory = NULL;

	if (ac - 1 < 1) return "Too few arguments";

	if (!strcmp(av[0], "exec") || !strcmp(av[0], "open")) {
		i = 1;
		self->ex_operation = strdup("open");
		if (self->ex_operation == NULL) return "Out of memory";
	} else if (!strcmp(av[0], "shellexecute")) {
		i = 2;
		self->ex_operation = strdup(av[1]);
		if (self->ex_operation == NULL) return "Out of memory";
	} else {
		return "Unknown command (BBCmd's bug)";
	}

	if (ac - 1 < i) return "Too few arguments";
	else {
		self->ex_file = strdup(av[i]);
		if (self->ex_file == NULL) return "Out of memory";
	}
	i++;

	if (ac - 1 < i) self->ex_parameters = NULL;
	else {
		self->ex_parameters = strdup(av[i]);
		if (self->ex_parameters == NULL) return "Out of memory";
	}
	i++;

	if (ac - 1 < i) self->ex_directory = NULL;
	else {
		self->ex_directory = strdup(av[i]);
		if (self->ex_directory == NULL) return "Out of memory";
	}
	i++;

	if (ac - 1 < i) self->ex_showcommand = SW_SHOWNORMAL;
	else {
		return "`showcmd' is not implemented yet.";
	}
	i++;

	return NULL;
}


static void
_BBCmdExecCommand_del(BBCmdCommand* _self)
{
	BBCmdExecCommand* self;

	assert(_self->cmd_descriptor == BBCmdExecCommandDescriptor);
	self = (BBCmdExecCommand*)_self;

	free(self->ex_operation);
	free(self->ex_file);
	free(self->ex_parameters);
	free(self->ex_directory);
}


static int
_BBCmdExecCommand_execute(const BBCmdCommand* _self)
{
	BBCmdExecCommand* self;

	assert(_self->cmd_descriptor == BBCmdExecCommandDescriptor);
	self = (BBCmdExecCommand*)_self;

	ShellExecute(
		NULL,
		self->ex_operation,
		self->ex_file,
		self->ex_parameters,
		self->ex_directory,
		self->ex_showcommand
	);

	return BBCmd_RC_TRUE;
}




static const char* _BBCmdExecCommand_alias[] = {
	"exec",
	"open",
	NULL
};

const BBCmdCommandDescriptor _BBCmdExecCommandDescriptor = {
	"shellexecute",
	_BBCmdExecCommand_alias,
	"(op, file [, params [, dir [, showcmd='shownormal']]]) --> true\n"
		"Perform an `op' (operation) on a specified `file'.\n"
		"`open(file [, ...])'\n"
		"is short for `shellexecute('open', file [, ...])'.\n"
		"`exec(file [, ...])' is same as `open(file [, ...])'.\n"
		"`showcmd' can not be specified.  It's not implemented yet.",
	sizeof(BBCmdExecCommand),
	_BBCmdExecCommand_initialize,
	_BBCmdExecCommand_del,
	_BBCmdExecCommand_execute
};




/* __END__ */
