/* Implementation of `focus' command */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <BBApi-C.h>

#include "Focus_p.h"




static const char*
_BBCmdFocusCommand_initialize( BBCmdCommand* _self,
                              int ac, const char* const* av,
                              const BBCmdCompiler* compiler )
{
	BBCmdFocusCommand* self;

	assert(_self->cmd_descriptor == BBCmdFocusCommandDescriptor);
	self = (BBCmdFocusCommand*)_self;

	self->fo_windowclass = NULL;
	self->fo_windowtitle = NULL;

	if (ac - 1 < 1 || 2 < ac - 1) return "Incorrect number of arguments";

	self->fo_windowclass = strdup(av[1]);
	if (self->fo_windowclass == NULL)
		return "Out of memory (_BBCmdFocusCommand_initialize)";

	if (ac - 1 == 2) {
		self->fo_windowtitle = strdup(av[2]);
		if (self->fo_windowtitle == NULL)
			return "Out of memory (_BBCmdFocusCommand_initialize)";
	}

	return NULL;
}


static void
_BBCmdFocusCommand_del(BBCmdCommand* _self)
{
	BBCmdFocusCommand* self;

	assert(_self->cmd_descriptor == BBCmdFocusCommandDescriptor);
	self = (BBCmdFocusCommand*)_self;

	free(self->fo_windowclass);
	free(self->fo_windowtitle);
}


static int
_BBCmdFocusCommand_execute(const BBCmdCommand* _self)
{
	BBCmdFocusCommand* self;
	HWND hwnd;
	HWND core;

	assert(_self->cmd_descriptor == BBCmdFocusCommandDescriptor);
	self = (BBCmdFocusCommand*)_self;

	hwnd = FindWindow(self->fo_windowclass, self->fo_windowtitle);
	if (hwnd == NULL) return BBCmd_RC_FALSE;

	/* Use the core if it's found */
	core = BBCmd_FindCore();
	if (core != NULL) {
		PostMessage(core, BB_BRINGTOFRONT, 0, (LPARAM)hwnd);
	} else {
		if (IsIconic(hwnd)) {
			PostMessage(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0);
		} else {
			/* # Switch workspace. */
			SetForegroundWindow(GetLastActivePopup(hwnd));
		}
	}

	return BBCmd_RC_TRUE;
}




static const char* _BBCmdFocusCommand_alias[] = {
	"activate",
	NULL
};

const BBCmdCommandDescriptor _BBCmdFocusCommandDescriptor = {
	"focus",
	_BBCmdFocusCommand_alias,
	"(class [, title])\n"
		"Activate the window specified by `class' and `title'.\n"
		"If it is not found, this command does nothing.",
	sizeof(BBCmdFocusCommand),
	_BBCmdFocusCommand_initialize,
	_BBCmdFocusCommand_del,
	_BBCmdFocusCommand_execute
};




/* __END__ */
