/* Implementation of misc. commands */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "MiscCmds_p.h"




/* ---------------- bbcmd_versionc ---------------- */

static const char*
_BBCmdVersionCommand_initialize( BBCmdCommand* _self,
                                 int ac, const char* const* av,
                                 const BBCmdCompiler* compiler )
{
	BBCmdVersionCommand* self;

	assert(_self->cmd_descriptor == BBCmdVersionCommandDescriptor);
	self = (BBCmdVersionCommand*)_self;

	if (ac - 1 != 0) return "Incorrect number of arguments";

	self->ve_output_to_console = !strcmp(av[0], "bbcmd_versionc");

	return NULL;
}


static void
_BBCmdVersionCommand_del(BBCmdCommand* _self)
{
	/* nothing to do */
}


static int
_BBCmdVersionCommand_execute(const BBCmdCommand* _self)
{
	BBCmdVersionCommand* self;
	char buf[80];
	char* release_level;

	assert(_self->cmd_descriptor == BBCmdVersionCommandDescriptor);
	self = (BBCmdVersionCommand*)_self;

	switch (BBCmd_RELEASE_LEVEL) {
	default: assert(0);
	case BBCmd_RELEASE_LEVEL_ALPHA: release_level = "a"; break;
	case BBCmd_RELEASE_LEVEL_BETA: release_level = "b"; break;
	case BBCmd_RELEASE_LEVEL_GAMMA: release_level = "c"; break;
	case BBCmd_RELEASE_LEVEL_FINAL: release_level = ""; break;
	}

	snprintf( buf, BBCmd_arraysizeof(buf), "%s %d.%d.%d%s",
	          "BBCmd",
	          BBCmd_MAJOR_VERSION,
	          BBCmd_MINOR_VERSION,
	          BBCmd_MICRO_VERSION,
	          release_level );
	buf[BBCmd_arraysizeof(buf) - 1] = '\0';

	if (self->ve_output_to_console)
		puts(buf);
	else {
		MessageBox( NULL, buf, "BBCmd",
			    MB_OK | MB_ICONINFORMATION | MB_TASKMODAL );
	}

	return BBCmd_RC_TRUE;
}




static const char* _BBCmdVersionCommand_alias[] = {
	"bbcmd_versionc",
	NULL
};

const BBCmdCommandDescriptor _BBCmdVersionCommandDescriptor = {
	"bbcmd_version",
	_BBCmdVersionCommand_alias,
	"() --> true\n"
		"Display the version of BBCmd as a dialog.\n"
		"`bbcmd_versionc' is like but display to stdout.",
	sizeof(BBCmdVersionCommand),
	_BBCmdVersionCommand_initialize,
	_BBCmdVersionCommand_del,
	_BBCmdVersionCommand_execute
};




/* ---------------- set_wallpaper ---------------- */

static const char*
_BBCmdSetWallpaperCommand_initialize( BBCmdCommand* _self,
                                      int ac, const char* const* av,
                                      const BBCmdCompiler* compiler )
{
	BBCmdSetWallpaperCommand* self;

	assert(_self->cmd_descriptor == BBCmdSetWallpaperCommandDescriptor);
	self = (BBCmdSetWallpaperCommand*)_self;

	self->sw_wallpaper = NULL;

	if (ac - 1 < 1) return "Too few arguments";
	if (2 < ac - 1) return "Too many arguments";

	self->sw_wallpaper = strdup(av[1]);
	if (self->sw_wallpaper == NULL)
		return "Out of memory (_BBCmdSetWallpaperCommand_initialize)";

	if (ac - 1 == 1)
		self->sw_style = 'c';
	else if (!strcmp(av[2], "center"))
		self->sw_style = 'c';
	else if (!strcmp(av[2], "full"))
		self->sw_style = 'f';
	else if (!strcmp(av[2], "tile"))
		self->sw_style = 't';
	else if (!strcmp(av[2], "F"))
		self->sw_style = 'F';
	else
		return "Invalid style";

	return NULL;
}


static void
_BBCmdSetWallpaperCommand_del(BBCmdCommand* _self)
{
	BBCmdSetWallpaperCommand* self;

	assert(_self->cmd_descriptor == BBCmdSetWallpaperCommandDescriptor);
	self = (BBCmdSetWallpaperCommand*)_self;

	if (self->sw_wallpaper != NULL)
		free(self->sw_wallpaper);
}


static int
_BBCmdSetWallpaperCommand_execute(const BBCmdCommand* _self)
{
	BBCmdSetWallpaperCommand* self;
	HKEY hkey;
	BYTE data[2];
	char wallpaper[MAX_PATH];

	assert(_self->cmd_descriptor == BBCmdSetWallpaperCommandDescriptor);
	self = (BBCmdSetWallpaperCommand*)_self;

	if ( _fullpath( wallpaper, self->sw_wallpaper,
	                BBCmd_arraysizeof(wallpaper) ) == NULL )
	{
		return BBCmd_RC_FALSE;
	}

	if (self->sw_style != 'F') {
		if ( RegOpenKeyEx( HKEY_CURRENT_USER,
		                   "Control Panel\\Desktop\\", 0,
		                   KEY_ALL_ACCESS, &hkey ) != ERROR_SUCCESS )
		{
			return BBCmd_RC_FALSE;
		}

		data[1] = '\0';
		data[0] = ((self->sw_style == 't') ? '1' : '0');
		RegSetValueEx(hkey, "TileWallpaper", 0, REG_SZ, data, 2);
		data[0] = ((self->sw_style == 'f') ? '2' : '0');
		RegSetValueEx(hkey, "WallpaperStyle", 0, REG_SZ, data, 2);
	}

	SystemParametersInfo( SPI_SETDESKWALLPAPER, 0, wallpaper,
	                      SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE );

	if (self->sw_style != 'F') {
		RegCloseKey(hkey);
	}

	return BBCmd_RC_TRUE;
}




const BBCmdCommandDescriptor _BBCmdSetWallpaperCommandDescriptor = {
	"set_wallpaper",
	NULL,
	"(bmp [, style='center'])\n"
		"Set `bmp' as the desktop wallpaper.\n"
		"`style' can be one of the following values:\n"
		"'center', 'full' or 'tile'.",
	sizeof(BBCmdSetWallpaperCommand),
	_BBCmdSetWallpaperCommand_initialize,
	_BBCmdSetWallpaperCommand_del,
	_BBCmdSetWallpaperCommand_execute
};




/* __END__ */
