/* Implementation of `random' command */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "Multi_p.h"
#include "Random_p.h"




static const char*
_BBCmdRandomCommand_initialize( BBCmdCommand* _self,
                                int ac, const char* const* av,
                                const BBCmdCompiler* compiler )
{
	/* deny to make a new instance from a compiler */
	abort();
	return "BUG (_BBCmdRandomCommand_initialize)";
}


static void
_BBCmdRandomCommand_del(BBCmdCommand* _self)
{
	BBCmdRandomCommand* self;

	assert(_self->cmd_descriptor == BBCmdRandomCommandDescriptor);
	self = (BBCmdRandomCommand*)_self;

	BBCmdCommand_Del((BBCmdCommand*)(self->ra_cmds));
}


static int
_BBCmdRandomCommand_execute(const BBCmdCommand* _self)
{
	BBCmdRandomCommand* self;
	int n;

	assert(_self->cmd_descriptor == BBCmdRandomCommandDescriptor);
	self = (BBCmdRandomCommand*)_self;

	if (self->ra_cmds->mu_list_size <= 0) return BBCmd_RC_TRUE;

	n = self->ra_cmds->mu_list_size * (rand() / (RAND_MAX + 1.0));

	return BBCmdCommand_Execute(self->ra_cmds->mu_list[n]);
}




const BBCmdCommandDescriptor _BBCmdRandomCommandDescriptor = {
	"random",
	NULL,
	"{cmd1 cmd2 ... cmdn} --> result_of_the_executed_command\n"
		"Built-in command to execute a command\n"
		"which is randomly selected from some given commands.",
	sizeof(BBCmdRandomCommand),
	_BBCmdRandomCommand_initialize,
	_BBCmdRandomCommand_del,
	_BBCmdRandomCommand_execute
};




BBCmdCommand*
BBCmdRandomCommand_New(BBCmdCommand* multicommand)
{
	BBCmdRandomCommand* self;
	int i;

	srand((unsigned)time(NULL));
	for (i = 0; i < 59; i++)
		(void)rand();    /* shuffle */

	if (multicommand->cmd_descriptor != BBCmdMultiCommandDescriptor)
		return NULL;

	self = malloc(sizeof(*self));
	if (self == NULL) return NULL;

	BBCmdCommand_INIT(self, BBCmdRandomCommandDescriptor);

	self->ra_cmds = (BBCmdMultiCommand*)multicommand;

	return (BBCmdCommand*)self;
}




/* __END__ */
