/**
 * `thread' command
 *
 * BUGS: Any instance of `thread' command assume that
 *       it is not used by some threads.
 */

#include <assert.h>
#include <stdlib.h>

#include "Thread_p.h"




static const char*
_BBCmdThreadCommand_initialize( BBCmdCommand* _self,
                                      int ac, const char* const* av,
                                      const BBCmdCompiler* compiler )
{
	/* deny to make a new instance from a compiler */
	abort();
	return "BUG (_BBCmdThreadCommand_initialize)";
}


static void
_BBCmdThreadCommand_del(BBCmdCommand* _self)
{
	BBCmdThreadCommand* self;

	assert(_self->cmd_descriptor == BBCmdThreadCommandDescriptor);
	self = (BBCmdThreadCommand*)_self;

	if (self->th_thread != NULL) {
		WaitForSingleObject(self->th_thread, INFINITE);
		CloseHandle(self->th_thread);
	}

	if (self->th_command != NULL)
		BBCmdCommand_Del(self->th_command);
}


static DWORD WINAPI
thread_proc(LPVOID lpparam)
{
	BBCmdThreadCommand* self;

	self = (BBCmdThreadCommand*)lpparam;

	BBCmdCommand_Execute(self->th_command);

	return 0;
}

static int
_BBCmdThreadCommand_execute(const BBCmdCommand* _self)
{
	BBCmdThreadCommand* self;
	DWORD thread_id;

	assert(_self->cmd_descriptor == BBCmdThreadCommandDescriptor);
	self = (BBCmdThreadCommand*)_self;

	if (self->th_thread != NULL) {
		DWORD state;

		state = WaitForSingleObject(self->th_thread, 0);
		if (state != WAIT_OBJECT_0)  /* is the thread still running? */
			return BBCmd_RC_FALSE;

		CloseHandle(self->th_thread);
	}

	self->th_thread = CreateThread( NULL,
	                                0,
	                                thread_proc,
	                                self,
	                                0,
	                                &thread_id );

	return (self->th_thread != NULL) ? BBCmd_RC_TRUE : BBCmd_RC_FALSE;
}




const BBCmdCommandDescriptor _BBCmdThreadCommandDescriptor = {
	"thread",
	NULL,
	"Built-in command to execute given command in new thread.",
	sizeof(BBCmdThreadCommand),
	_BBCmdThreadCommand_initialize,
	_BBCmdThreadCommand_del,
	_BBCmdThreadCommand_execute
};




BBCmdCommand*
BBCmdThreadCommand_New(BBCmdCommand* command)
{
	BBCmdThreadCommand* self;

	self = malloc(sizeof(*self));
	if (self == NULL) return NULL;

	BBCmdCommand_INIT(self, BBCmdThreadCommandDescriptor);

	self->th_thread = NULL;
	self->th_command = command;

	return (BBCmdCommand*)self;
}




/* __END__ */
