/* The tool to list up available commands */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "BBCmd.h"





int
numof(const char* s, char c)
{
	const char* p;
	int n;

	p = s;
	n = 0;

	while (*p != '\0') {
		if (*p == c)
			n++;
		p++;
	}

	return n;
}


void
strrep(char* dest, const char* orig, char pat, const char* rep)
{
	const char* i;
	char* o;
	const char* r;

	i = orig;
	o = dest;
	while (*i != '\0') {
		if (*i == pat) {
			r = rep;
			while (*r != '\0') {
				*o = *r;
				o++;  r++;
			}
			i++;
		} else {
			*o = *i;
			o++;  i++;
		}
	}
	*o = '\0';
}


int
compare_descriptor(const void* _a, const void* _b)
{
	const BBCmdCommandDescriptor* a;
	const BBCmdCommandDescriptor* b;

	a = *(const BBCmdCommandDescriptor**)_a;
	b = *(const BBCmdCommandDescriptor**)_b;

	return strcmp( BBCmdCommandDescriptor_NameOf(a),
	               BBCmdCommandDescriptor_NameOf(b) );
}




int
main(void)
{
	BBCmdCompiler* compiler;
	const BBCmdCommandDescriptor* const* d_org;
	const BBCmdCommandDescriptor** d;
	int n;
	int i;
	int r = 0;

	compiler = BBCmdCompiler_New();
	if (compiler == NULL) {puts("Out of memory");  r = !0;  goto ERR1;}

	d_org = BBCmdCompiler_AvailableCommand(compiler, &n);
	d = (const BBCmdCommandDescriptor**)malloc(sizeof(*d) * n);
	if (d == NULL) {puts("Out of memory");  r = !0;  goto ERR2;}

	memcpy(d, d_org, sizeof(*d) * n);
	qsort(d, n, sizeof(*d), compare_descriptor);

	for (i = 0; i < n; i++) {
		const char* const* a;
		const char* doc;
		char* s;
		int len;
		int n;

		printf("%s", BBCmdCommandDescriptor_NameOf(d[i]));

		doc = BBCmdCommandDescriptor_DocOf(d[i]);
		len = strlen(doc);
		n = numof(doc, '\n');
		s = malloc(sizeof(char) * (len+1+n));
		if (s == NULL) {puts("Out of memory");  r = !0;  goto ERR3;}
		strrep(s, doc, '\n', "\n\t");
		printf("%s\n", s);
		free(s);

		a = BBCmdCommandDescriptor_AliasOf(d[i]);
		if (a != NULL && *a != NULL) {
			int len;

			len = 8 + 5 + 1 + 1 + strlen(*a);
			printf("\n\tAlias: %s", *a);
			a++;
			while (*a != NULL) {
				len += 2 + strlen(*a);
				if (len < 60) {
					printf(", %s", *a);
				} else {
					if (*(a + 1) != NULL)
						printf(",");
					printf("\n\t       %s", *a);
					len = 8 + 5 + 1 + 1 + strlen(*a);
				}
				a++;
			}
			printf("\n");
		}
		printf("\n");
	}

ERR3:
	free(d);
ERR2:
	BBCmdCompiler_Del(compiler);
ERR1:
	return r;
}




/* __END__ */
